package logic

import (
	"context"
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"time"

	"gorm.io/gorm"

	"fusenapi/server/canteen/internal/svc"
	"fusenapi/server/canteen/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type SaveCanteenTypeProductLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewSaveCanteenTypeProductLogic(ctx context.Context, svcCtx *svc.ServiceContext) *SaveCanteenTypeProductLogic {
	return &SaveCanteenTypeProductLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 保存餐厅类型的关联产品
func (l *SaveCanteenTypeProductLogic) SaveCanteenTypeProduct(req *types.SaveCanteenTypeProductReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	if userinfo.GetIdType() != auth.IDTYPE_User {
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "please sign in first")
	}
	if len(req.ProductList) == 0 {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "product list can`t be empty")
	}
	canteenProductModel := gmodel.NewFsCanteenProductModel(l.svcCtx.MysqlConn)
	//获取原有餐厅类型的所有产品
	oldCanteenProductList, err := canteenProductModel.GetAllByCanteenTypeId(l.ctx, req.Id)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "failed to get canteen product list")
	}
	sizeIds := make([]int64, 0, len(req.ProductList))
	for _, v := range req.ProductList {
		sizeIds = append(sizeIds, v.SizeId)
	}
	productSizeModel := gmodel.NewFsProductSizeModel(l.svcCtx.MysqlConn)
	productSizeList, err := productSizeModel.GetAllByIds(l.ctx, sizeIds, "")
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "failed to get product size list")
	}
	mapProductSize := make(map[int64]gmodel.FsProductSize)
	for _, v := range productSizeList {
		mapProductSize[v.Id] = v
	}
	now := time.Now().UTC().Unix()
	//开启事务
	err = l.svcCtx.MysqlConn.Transaction(func(tx *gorm.DB) error {
		canteenProductModel = gmodel.NewFsCanteenProductModel(tx)
		sort := int64(0)
		//新的变更记录
		mapUpdateCanteenPid := make(map[int64]struct{})
		for _, v := range req.ProductList {
			sort++
			sizeInfo, ok := mapProductSize[v.SizeId]
			if !ok {
				continue
			}
			if v.Id > 0 { //更新
				mapUpdateCanteenPid[v.Id] = struct{}{}
				err = canteenProductModel.UpdateById(l.ctx, v.Id, &gmodel.FsCanteenProduct{
					SizeId:    &v.SizeId,
					Sid:       &v.SId,
					Sort:      &sort,
					ProductId: sizeInfo.ProductId,
				})
				if err != nil {
					return err
				}
				continue
			}
			//新增
			addStatus := int64(1)
			err = canteenProductModel.Create(l.ctx, &gmodel.FsCanteenProduct{
				SizeId:      &v.SizeId,
				Sid:         &v.SId,
				Sort:        &sort,
				Status:      &addStatus,
				Ctime:       &now,
				CanteenType: &req.Id,
				ProductId:   sizeInfo.ProductId,
			})
			if err != nil {
				return err
			}
		}
		diffCanteenProductId := make([]int64, 0, len(oldCanteenProductList))
		//旧的中不包含在更新的里面则去掉
		for _, v := range oldCanteenProductList {
			if _, ok := mapUpdateCanteenPid[v.Id]; !ok {
				diffCanteenProductId = append(diffCanteenProductId, v.Id)
			}
		}
		if len(diffCanteenProductId) == 0 {
			return nil
		}
		delStatus := int64(0)
		if err = canteenProductModel.UpdateByIdArr(l.ctx, diffCanteenProductId, &gmodel.FsCanteenProduct{
			Status: &delStatus,
		}); err != nil {
			return err
		}
		return nil
	})
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, "failed to save canteen product")
	}
	return resp.SetStatus(basic.CodeOK)
}
