package logic

import (
	"context"
	"encoding/json"
	"fusenapi/model/gmodel"
	"fusenapi/server/product-template-tag/internal/svc"
	"fusenapi/server/product-template-tag/internal/types"
	"fusenapi/service/repositories"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"fusenapi/utils/s3url_to_s3id"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetProductTemplateTagsLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetProductTemplateTagsLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetProductTemplateTagsLogic {
	return &GetProductTemplateTagsLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *GetProductTemplateTagsLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *GetProductTemplateTagsLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }
type LogoSelect struct {
	LogoSelected struct {
		LogoSelectedId      int `json:"logo_selected_id"`
		TemplateTagSelected struct {
			TemplateTag   string     `json:"template_tag"`
			Color         [][]string `json:"color"`
			SelectedIndex int        `json:"selected_index"`
		} `json:"template_tag_selected"`
	} `json:"logo_selected"`
}

func (l *GetProductTemplateTagsLogic) GetProductTemplateTags(req *types.GetProductTemplateTagsReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	if req.Limit <= 0 || req.Limit > 100 {
		req.Limit = 5
	}
	var (
		productTemplateTags []gmodel.FsProductTemplateTags
		err                 error
	)
	//获取用户需要渲染logo
	logoInfo, err := l.svcCtx.Repositories.ImageHandle.LogoInfo(l.ctx, &repositories.LogoInfoReq{
		UserId:  userinfo.UserId,
		GuestId: userinfo.GuestId,
	})
	if err != nil {
		logx.Error(err)
		return
	}
	if logoInfo.Metadata == nil || *logoInfo.Metadata == "" {
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "logo info`s metadata is not set")
	}
	//解析用户素材元数据
	var metaData map[string]interface{}
	if err = json.Unmarshal([]byte(*logoInfo.Metadata), &metaData); err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeJsonErr, "failed to parse user metadata")
	}
	//解析模板标签颜色
	var mapMaterialTemplateTagColors map[string][][]string
	b, _ := json.Marshal(metaData["template_tag"])
	if err = json.Unmarshal(b, &mapMaterialTemplateTagColors); err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeJsonErr, "invalid format of metadata`s template_tag")
	}
	//解析单纯的模板标签用于排序
	var simpleTemplateTags []string
	b, _ = json.Marshal(metaData["template_tag_id"])
	if err = json.Unmarshal(b, &simpleTemplateTags); err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeJsonErr, "invalid format of metadata`s template_tag_id")
	}
	//从用户元数据获取选中的颜色数据
	mapSelectColor := make(map[string]int) //key是模板标签val是选中的索引
	if logoInfo.UserInfoMetadata != nil && *logoInfo.UserInfoMetadata != "" {
		//解析用户信息元数据
		var logoSelectInfo LogoSelect
		if err = json.Unmarshal([]byte(*logoInfo.UserInfoMetadata), &logoSelectInfo); err != nil {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeJsonErr, "failed to parse user info metadata")
		}
		//设置选中
		key := logoSelectInfo.LogoSelected.TemplateTagSelected.TemplateTag
		if _, ok := mapMaterialTemplateTagColors[key]; ok {
			mapSelectColor[key] = logoSelectInfo.LogoSelected.TemplateTagSelected.SelectedIndex
		}
	}
	var templateTagNameList []string
	for templateTag, _ := range mapMaterialTemplateTagColors {
		templateTagNameList = append(templateTagNameList, templateTag)
	}
	productTemplateTags, err = l.svcCtx.AllModels.FsProductTemplateTags.GetListByTagNames(l.ctx, templateTagNameList, len(templateTagNameList), 1, "id DESC")
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get template tags")
	}
	//资源id集合
	resourceIds := make([]string, 0, 5)
	for _, v := range productTemplateTags {
		resourceIds = append(resourceIds, s3url_to_s3id.GetS3ResourceIdFormUrl(*v.Cover))
	}
	//根据resourceUrls找到对应的元数据
	resourceMetadataList, err := l.svcCtx.AllModels.FsResource.FindAllByResourceIds(l.ctx, resourceIds)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get resource list")
	}
	mapResourceMetadata := make(map[string]map[string]interface{})
	for _, v := range resourceMetadataList {
		var metadata map[string]interface{}
		if v.Metadata != nil {
			_ = json.Unmarshal(*v.Metadata, &metadata)
		}
		mapResourceMetadata[*v.ResourceUrl] = metadata
	}
	//排序
	for index, templateTagStr := range simpleTemplateTags {
		for k, v := range productTemplateTags {
			if templateTagStr == *v.TemplateTag {
				productTemplateTags[k], productTemplateTags[index] = productTemplateTags[index], productTemplateTags[k]
				break
			}
		}
	}
	list := make([]types.GetProductTemplateTagsRsp, 0, len(productTemplateTags))
	for _, templateTagInfo := range productTemplateTags {
		colors := make([][]string, 0, 10)
		SelectedColorIndex := 0
		isDefaultTemplateTag := false
		//查看用户素材中标签对应的颜色
		if colorsSet, ok := mapMaterialTemplateTagColors[*templateTagInfo.TemplateTag]; ok {
			//是不是选中的标签
			if selectIndex, ok := mapSelectColor[*templateTagInfo.TemplateTag]; ok {
				isDefaultTemplateTag = true
				//标签中选中的索引
				SelectedColorIndex = selectIndex
			}
			//标签颜色
			colors = colorsSet
		}
		var templateTagGroups []interface{}
		if templateTagInfo.Groups != nil && *templateTagInfo.Groups != "" {
			if err = json.Unmarshal([]byte(*templateTagInfo.Groups), &templateTagGroups); err != nil {
				logx.Error(err)
				return resp.SetStatusWithMessage(basic.CodeJsonErr, "failed to parse groups")
			}
		}
		list = append(list, types.GetProductTemplateTagsRsp{
			Id:                   templateTagInfo.Id,
			TemplateTag:          *templateTagInfo.TemplateTag,
			IsDefaultTemplateTag: isDefaultTemplateTag,
			TemplateTagGroups:    templateTagGroups,
			Cover:                *templateTagInfo.Cover,
			CoverMetadata:        mapResourceMetadata[*templateTagInfo.Cover],
			Colors:               colors,
			SelectedColorIndex:   SelectedColorIndex,
		})
	}
	return resp.SetStatusWithMessage(basic.CodeOK, "success", list)
}
