package logic

import (
	"context"
	"encoding/json"
	"errors"
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"fusenapi/utils/s3url_to_s3id"
	"gorm.io/gorm"

	"fusenapi/server/product-template-tag/internal/svc"
	"fusenapi/server/product-template-tag/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetTemplateTagColorLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetTemplateTagColorLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetTemplateTagColorLogic {
	return &GetTemplateTagColorLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *GetTemplateTagColorLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

func (l *GetTemplateTagColorLogic) GetTemplateTagColor(req *types.GetTemplateTagColorReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	if req.SelectedColorIndex < 0 {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "param selected_color_index is invalid")
	}
	//根据logo查询素材资源
	resourceId := s3url_to_s3id.GetS3ResourceIdFormUrl(req.Logo)
	if resourceId == "" {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "param logo is invalid")
	}
	var (
		userMaterial    *gmodel.FsUserMaterial
		templateTagInfo *gmodel.FsProductTemplateTags
		err             error
	)
	//获取模板标签信息
	templateTagInfo, err = l.svcCtx.AllModels.FsProductTemplateTags.FindOneByTagName(l.ctx, req.TemplateTag)
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return resp.SetStatusWithMessage(basic.CodeDbRecordNotFoundErr, "the template tag is not exists")
		}
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get template tag info")
	}
	userMaterial, err = l.svcCtx.AllModels.FsUserMaterial.FindOneByLogoResourceId(l.ctx, resourceId)
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			return resp.SetStatusWithMessage(basic.CodeDbRecordNotFoundErr, "the logo is not found")
		}
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get user material")
	}
	if userMaterial.Metadata == nil || len(*userMaterial.Metadata) == 0 {
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "the user material is empty")
	}
	//解析用户素材元数据
	var metaData map[string]interface{}
	if err = json.Unmarshal(*userMaterial.Metadata, &metaData); err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeJsonErr, "failed to parse user metadata")
	}
	var mapMaterialTemplateTag map[string][][]string
	b, _ := json.Marshal(metaData["template_tag"])
	if err = json.Unmarshal(b, &mapMaterialTemplateTag); err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeJsonErr, "invalid format of metadata`s template_tag")
	}
	colors, ok := mapMaterialTemplateTag[req.TemplateTag]
	if !ok {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "the template tag is not found from this logo material`s metadata")
	}
	if req.SelectedColorIndex >= len(colors) {
		return resp.SetStatusWithMessage(basic.CodeRequestParamsErr, "select color index is out of range !!")
	}
	var templateTagGroups interface{}
	if templateTagInfo.Groups != nil && *templateTagInfo.Groups != "" {
		if err = json.Unmarshal([]byte(*templateTagInfo.Groups), &templateTagGroups); err != nil {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeJsonErr, "failed to parse template tag`s groups info")
		}
	}
	return resp.SetStatusWithMessage(basic.CodeOK, "success", types.GetTemplateTagColorRsp{
		Colors:             colors,
		SelectedColorIndex: req.SelectedColorIndex,
		TemplateTagGroups:  templateTagGroups,
	})
}

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *GetTemplateTagColorLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }
